package k2;

import java.awt.Image;
import java.awt.Rectangle;
import javax.swing.ImageIcon;

/**
 * The Button class is a simple model of the geometry and function of a
 * button on the Elecraft K2 transceiver.  The model additionally deals with
 * phantom buttons.  These are buttons that appear when the mouse moves over
 * certain areas on the front panel and disappear when the mouse moves out of
 * the area.  Phantom buttons have images and image locations associated with
 * them.
 *
 * <p>Mike Markowski, ab3ap@arrl.net<br>
 * March 2007
 */

public class Button {
	
	Image phantomImage; // Picture of pop up phantom button.
	int pX, pY; // Location, in pixels, of phantom button image.
	Rectangle geometry; // Geometry in pixels of this button.
	String primary; // K2 command for this button's primary function.
	String secondary; // K2 command for this button's secondary function.

	/**
	 * The Button constructor uses K2 functions and button geometry.
	 * The first arguments, fn1 and fn2, are the K2 command strings needed to
	 * indicate a button tap or hold, respectively.  The remaining arguments
	 * describe where on the image, in pixels, is this particular button.
	 */

	Button(String fn1, String fn2, int x0, int y0, int xlen, int ylen) {
		primary = fn1;
		secondary = fn2;
		geometry = new Rectangle(x0, y0, xlen, ylen);
		phantomImage = null;
	}

	/** Determines if the button contains the specified point, in units
	of pixels. */

	public boolean contains(int x, int y) {
		return geometry.contains(x, y);
	}

	/** Return the buttons image location and size, in pixels. */
	public Rectangle getGeometry() { return geometry; }
	/** Specify the buttons image location and size, in pixels. */
	public void setGeometry(Rectangle r) { geometry = r; }
	/** Specify the buttons image location and size, in pixels. */
	public void setGeometry(int x0, int y0, int xlen, int ylen) {
		geometry = new Rectangle(x0, y0, xlen, ylen);
	}

	/** Return the K2 command string associated with a button tap. */
	public String getTapFn() { return primary; }
	/** Specify the K2 command string associated with a button tap. */
	public void setTapFn(String fn) { primary = fn; }

	/** Return the K2 command string associated with a button hold. */
	public String getHoldFn() { return secondary; }
	/** Specify the K2 command string associated with a button hold. */
	public void setHoldFn(String fn) { secondary = fn; }

	/** Return the Image of the phantom button. */
	public Image getPhantomImage() { return phantomImage; }
	/** Return the x coordinate, in pixels, of this phantom button. */
	public int getPhantomX() { return pX; }
	/** Return the y coordinate, in pixels, of this phantom button. */
	public int getPhantomY() { return pY; }
	/** Specify the image file's path and in-image location, in pixels,
	of this phantom button. */
	public void setPhantomImage(String filename, int x, int y) {
		phantomImage = createImage(filename, "");
		pX = x;
		pY = y;
	}
	/** Answer if this button is a phantom button or not. */
	public boolean isPhantomButton() { return phantomImage != null; }

    /** Returns an Image, or null if the path was invalid.  This is the image
	 * that will be painted on the screen when the mouse moves into the
	 * phatom button's area specified by its geometry.  */
	private static Image createImage(String path, String description) {

		ImageIcon icon = createImageIcon(path);
		if (icon == null)
			return null;
		return icon.getImage();

	}

    /** Returns an ImageIcon, or null if the path was invalid. */
	private static ImageIcon createImageIcon(String path) {

		java.net.URL imgURL = Button.class.getResource(path);
		if (imgURL != null)
			return new ImageIcon(imgURL);
		else
			System.err.println("Couldn't find file: " + path);
		return null;
	}


}
