package k2;

import java.awt.Point;

/** The Knob object describes the geometry of a knob as well as provides
 * methods to determine if a knob contains a point, direction of spin, and
 * if it's in use.  This class is further extended to create particular knobs
 * to simulate the functions of a K2.
 *
 * <p>Mike Markowski, ab3ap@arrl.net<br>
 * March 2007
 */

public class Knob {

	Point setPoint;
	boolean inUse; // Is knob currently being turned.
	double stepsMoved;
	double v0x, v0y, v1x, v1y; // Previous (v0) and current (v1) settings.
	int ctrX, ctrY, radius; // Center and radius, in pixels, of knob.
	double min, max; // Min and max setting, in real world units (wpm, W, etc.)
	int minStep, maxStep; // Min and max setting, in knob steps [0, numSteps).
	double setting; // Current setting, in real world units.
	int numSteps; // Number of discrete steps on this knob.
	int turningDir; // 1 is CW, -1 is CCW.

	Knob() { this(0, 0, 1); }

	/** Make a knob with specified center and radius, in units of pixels. */
	Knob(int cx, int cy, int r) {
		ctrX = cx;
		ctrY = cy;
		radius = r;
		inUse = false;
		setting = 0;
		stepsMoved = 0.;
		turningDir = 1;
		setPoint = new Point(0, 0);
	}

	/** Return knob's current setting, initially 0. */
	public double getSetting() { return setting; }
	/** Set knob's current setting. */
	public void setSetting(double s) {
		setting = s;
		setting = (setting < min) ? min : setting;
		setting = (setting > max) ? max : setting;
	}

	/** Return knob's minimum setting. */
	public double getMin() { return min; }
	/** Set knob's minimum setting. */
	public void setMin(double m) { min = m; }

	/** Return knob step associated with minimum setting. */
	public int getMinStep() { return minStep; }
	/** Set knob step associated with minimum setting. */
	public void setMinStep(int m) { minStep = m; }

	/** Return knob's maximum setting. */
	public double getMax() { return max; }
	/** Set knob's maximum setting. */
	public void setMax(double m) { max = m; }

	/** Return knob step associated with maximum setting. */
	public int getMaxStep() { return maxStep; }
	/** Set knob step associated with maximum setting. */
	public void setMaxStep(int m) { maxStep = m; }

	/** Return x coordinate of knob's center. */
	public int getX() { return ctrX; }
	/** Return y coordinate of knob's center. */
	public int getY() { return ctrY; }
	/** Return radius of knob. */
	public int getRadius() { return radius; }

	/** Return the number of discrete steps per revolution modeled by this
	 * knob. */
	public int getSteps() { return numSteps; }
	/** Set number of steps per revolution. */
	public void setSteps(int s) { numSteps = s; }

	/** Answers if this knob is currently in use, i.e., being turned by
	user. */
	public boolean inUse() { return inUse; }
	/** Specify whether this knob is currently in use. */
	public void setInUse(boolean b) { inUse = b; }

	/** Return current setting.  This simulates a knob that runs from
	 * -45 degrees off the lower vertical to 45 degrees off the same
	 * vertical. */
	public double calcSetting() {
		double angle = Math.acos(-v0y);
		double val = angle/(2.*Math.PI) * numSteps;
		// Adjust setting since acos is in [0, pi], not [0, 2pi].
		if (v0x > 0)
			val += numSteps/2;
		else
			val = numSteps/2 - val;
		setting = (int) val;
		setting = (setting < min) ? min : setting;
		setting = (setting > max) ? max : setting;
		return setting;
	}

	/** Calculate the point, in pixels, of where the current knob setting, in
	 * real world coordinates, lies on the uter edge of the knob.  A line
	 * will be drawn fro the knob center to this point to simulate the white
	 * painted line on the real knob. */
	public void calcSetPoint() {
		double s = (setting - min) / (max - min); // Fraction of range.
		// Knob step corresponding to setting.
		s = s*(maxStep - minStep) + minStep;
		// Convert step location to angle in radians.
		// Next line is reduced: s = 2*Math.PI * s/numSteps + Math.PI/2;
		s = Math.PI * (2*s/numSteps + 0.5);
		setPoint.setLocation((int) (radius * Math.cos(s)),
			(int) (radius * Math.sin(s)));
		setPoint.translate(ctrX, ctrY);
	}

	public Point getSetPoint() { return setPoint; }

	/** Calculate the angle, in radians, between the two most recent points
	as specified by calls to setCurrentSetting(). */
	public double theta() {
		// Ue dot product definition to get angle between.  In [0, pi].
		return Math.acos(v0x*v1x + v0y*v1y);
	}

	/** Get the current knob turning direction, where 1 is clockwise and -1
	 * is counter-clockwise. */
	public int getTurningDir() { return turningDir; }
	/** Set the current knob turning direction, where 1 is clockwise and -1
	 * is counter-clockwise. */
	public void setTurningDir(int dir) { turningDir = dir; }
	/** Calculate direction this knob is being spun, 1 for clockwise and -1
	 for counter-clockwise. */
	private int calcTurningDir() {
		// Find just z component of cross product to turning direction.
		// Return 1 (CW) or -1 (CCW).
		return (v1x*v0y - v0x*v1y < 0) ? -1 : 1;
	}

	/** Specify a new knob set point.  This point is used relative to the
	 knob center to create a unit vector pnoiting from knob center outwards.
	 The most recent previous setting is also saved for used with vector
	 calculations in theta() and turningDir(). */
	public void setCurrentSetting(int x, int y) {
		// Save current setting as previous setting.
		v1x = v0x;
		v1y = v0y;
		// Save new point as current setting and convert to unit vector.
		double mag = distance(ctrX, ctrY, x, y);
		v0x = (x - ctrX) / mag;
		v0y = (y - ctrY) / mag;
		turningDir = calcTurningDir();
	}

	/** Indicate whether or not point (x, y), in pixels, is within the circle
	 bounding the knob. */
	public boolean contains(int x, int y) {
		return distance(x, y, ctrX, ctrY) <= radius;
	}

	/** Distance between two points. */
	private double distance(double x0, double y0, double x1, double y1) {
		return Math.sqrt((x1-x0)*(x1-x0) + (y1-y0)*(y1-y0));
	}

	/** How many steps has knob moved since last update? */
	public int getStepsMoved() { return (int) stepsMoved; }
	public void setStepsMoved(double s) { stepsMoved = s; }

	/** Count stpes moved since last update. */
	public void countStepsMoved() {
		// Look at two most recent setting, v0 & v1.  Turn the angle
		// between into a number in [0, numSteps] because we want to simulate
		// the VFO knob having numSteps steps per revolution.
		stepsMoved += theta()/(2*Math.PI) * numSteps;
	}

	/** Once program has made use of number of steps moved we want to clear
	 * the counter.  However, we save the mantissa to apply towards future
	 * knob spins. */
	public void clearSteps() {
		// Save unused fractional amount of spin for next use.
		stepsMoved -= (int) stepsMoved;
	}

}
