
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Observable;
import util.Serial;

/**
 * This is a super simple program that simply sends commands to the K2 over the
 * serial link and prints the results to stdout. These are the values that you
 * will want to save in case your K2 is ever in need of a complete reset or
 * similar memory catastrophe.
 *
 * Run from the command line by typing:
 *
 * java Param or java Param COM1 or whatever serial port you're using. The
 * default is /dev/ttyS0.
 *
 * Mike Markowski, ab3ap@arrl.net April 2007
 */
public class K2Reader extends Observable implements Runnable {

    private boolean stopReading;
    private int k2cmds; // # K2 commands sent.
    private int k2cmdsTot;
    private File outputFile;
    private Serial sport;

    K2Reader(Serial sport) {
        this.sport = sport;
        k2cmds = 0;
        k2cmdsTot = 213;
        stopReading = false;
    }

    public void setOutputFile(File outputFile) {
        this.outputFile = outputFile;
    }

    public void setStopReading(boolean stopReading) {
        this.stopReading = stopReading;
    }

    private void readSettings() {

        boolean haveKDSP2 = false;

        BufferedWriter out;
        try {
            out = new BufferedWriter(new FileWriter(outputFile));
        } catch (IOException ex) {
            System.out.println("Couldn't open " + outputFile.getAbsoluteFile()
                    + " for writing.");
            return;
        }

        varVal("K22;", false);
        for (int i = 38; i < 70; i++) {
            if (i == 51) {
                continue; // Skip over CAL settings for now.
            }
            String command = "SW" + i + ";SW05;";
            String s = varVal(command, true);
            s = s.trim();
            if (s.equals("RTC ON")) {
                haveKDSP2 = true;
                k2cmdsTot = 293;
            }
            qWriteln(out, s);
            varVal("SW05;", false);
        }

        qWriteln(out, "\nCAL");
        // Get CAL CUR setting.
        varVal("SW51;SW05;", false); // CAL, MENU
        varVal("SW21;SW01;", false); // EDIT, BAND+
        varVal("SW01;", false); // BAND+, EDIT displays max current.
        qWrite(out, "  ");
        qWriteln(out, varVal("SW21;", true));
        varVal("SW05;", false);
        // Get CAL TPA setting.
        varVal("SW51;SW05;", false); // CAL, MENU
        varVal("SW21;SW01;", false); // EDIT, BAND+
        varVal("SW01;SW01;", false); // BAND+, BAND+
        qWrite(out, "  ");
        qWriteln(out, varVal("SW21;", true));
        varVal("SW05;", false);
        // Get CAL S LO setting.
        varVal("SW51;SW05;", false); // CAL, MENU
        varVal("SW21;SW01;", false); // EDIT, BAND+
        varVal("SW01;SW01;", false); // BAND+, BAND+
        varVal("SW01;", false); // BAND+, EDIT
        qWrite(out, "  ");
        qWriteln(out, varVal("SW21;", true));
        varVal("SW05;", false);
        // Get CAL S LO setting.
        varVal("SW51;SW05;", false); // CAL, MENU
        varVal("SW21;SW01;", false); // EDIT, BAND+
        varVal("SW01;SW01;", false); // BAND+, BAND+
        varVal("SW01;SW01;", false); // BAND+, BAND+
        qWrite(out, "  ");
        qWriteln(out, varVal("SW21;", true));
        varVal("SW05;", false);

        qWriteln(out, "\nFilter settings");
        // Go to 7.050 MHz.
        varVal("FA00007050000;", false);
        // CW mode.
        qWrite(out, String.format("  %-10s", "CW:"));
        varVal("MD3;", false);
        getFilterSettings(out);
        // CW reverse mode.
        qWrite(out, String.format("  %-10s", "CW rev:"));
        varVal("MD7;", false);
        getFilterSettings(out);
        // LSB mode.
        qWrite(out, String.format("  %-10s", "LSB:"));
        varVal("MD1;", false);
        getFilterSettings(out);
        // USB mode.
        qWrite(out, String.format("  %-10s", "USB:"));
        varVal("MD2;", false);
        getFilterSettings(out);
        // RTTY mode.
        qWrite(out, String.format("  %-10s", "RTTY:"));
        varVal("MD6;", false);
        getFilterSettings(out);
        // RTTY reverse mode.
        qWrite(out, String.format("  %-10s", "RTTY rev:"));
        varVal("MD9;", false);
        getFilterSettings(out);

        qWriteln(out, "\nBFO settings");
        // Go to 7.050 MHz.
        varVal("FA00007050000;", false);
        // CW mode.
        qWrite(out, String.format("  %-10s", "CW:"));
        varVal("MD3;", false);
        getBFOSettings(out);
        // CW reverse mode.
        qWrite(out, String.format("  %-10s", "CW rev:"));
        varVal("MD7;", false);
        getBFOSettings(out);
        // LSB mode.
        qWrite(out, String.format("  %-10s", "LSB:"));
        varVal("MD1;", false);
        getBFOSettings(out);
        // USB mode.
        qWrite(out, String.format("  %-10s", "USB:"));
        varVal("MD2;", false);
        getBFOSettings(out);
        // RTTY mode.
        qWrite(out, String.format("  %-10s", "RTTY:"));
        varVal("MD6;", false);
        getBFOSettings(out);
        // RTTY reverse mode.
        qWrite(out, String.format("  %-10s", "RTTY rev:"));
        varVal("MD9;", false);
        getBFOSettings(out);

        if (haveKDSP2) {
            // DSP settings.
            qWriteln(out, "\nDSP settings");
            // CW.
            getDSPSettings(out);
        }
        setChanged();
        notifyObservers(new Integer(100));

        try {
            out.close();
        } catch (IOException ex) {
            System.out.println("Couldn't close file "
                    + outputFile.getAbsoluteFile());
        }

    }

    public void getFilterSettings(BufferedWriter out) {
        String s;
        // Start with filter FL1.
        varVal("FW00001;", false);
        // Get filter setting.
        varVal("SW51;SW05;", false);
        varVal("SW21;SW03;", false);
        s = varVal("SW21;", true); // FL1.
//        varVal("SW05;", false);
        qWrite(out, String.format("%-12s", s));
        s = varVal("SW13;", true); // FL2.
        qWrite(out, String.format("%-12s", s));
        s = varVal("SW13;", true); // FL3.
        qWrite(out, String.format("%-12s", s));
        s = varVal("SW13;", true); // FL4.
        qWriteln(out, String.format("%-12s", s));
        varVal("FW00001;", false);
    }

    public void getBFOSettings(BufferedWriter out) {
        String s;
        // Start with filter FL1.
        varVal("FW00001;", false);
        // Get filter setting.
        varVal("SW51;SW05;", false); // CAL, MENU
        varVal("SW21;SW03;", false); // EDIT, DOWN
        varVal("SW21;", false); // FL1
        varVal("SW03;", false); // BAND-
        s = varVal("SW02;", true, 1500); // DISPLAY
        qWrite(out, String.format("%-12s", s));
        s = varVal("SW13;", true, 1500); // FL2.
        qWrite(out, String.format("%-12s", s));
        s = varVal("SW13;", true, 1500); // FL3.
        qWrite(out, String.format("%-12s", s));
        s = varVal("SW13;", true, 1500); // FL4.
        qWriteln(out, String.format("%-12s", s));
        varVal("FW00001;", false);
        varVal("SW05;", true); // MENU
    }

    public void getDSPSettings(BufferedWriter out) {
        String s;
        for (int j = 0; j < 3; j++) {
            String cmd = null;
            switch (j) {
                case 0:
                    cmd = "MD3;";
                    break;
                case 1:
                    cmd = "MD1;";
                    break;
                case 2:
                    cmd = "MD6;";
                    break;
            }
            varVal(cmd, false);
            for (int i = 0; i < 4; i++) {
                s = varVal("SW02;", true);
                qWriteln(out, String.format("  %-12s", s));
                if (j != 0) {
                    s = varVal("SW17;", true);
                    qWriteln(out, String.format("  %-12s", s));
                }
                if (j != 2) {
                    s = varVal("SW17;", true);
                    qWriteln(out, String.format("  %-12s", s));
                    s = varVal("SW17;", true);
                    qWriteln(out, String.format("  %-12s", s));
                    s = varVal("SW17;", true);
                    qWriteln(out, String.format("  %-12s", s));
                    s = varVal("SW17;", true);
                    qWriteln(out, String.format("  %-12s", s));
                }
                s = varVal("SW17;", true);
                qWriteln(out, String.format("  %-12s\n", s));
                varVal("SW29;", false);
            }
        }
    }

    public String parseDS(byte ds[]) {
        String display = "";
        for (int i = 2; i < 10; i++) {
            boolean dp = ((ds[i] & 0x80) != 0);
            if (dp) {
                display += ".";
            }
            char b = (char) (ds[i] & 0x7f); // Clear MSBit.
            switch (b) {
                case '<':
                    display += 'L';
                    break;
                case '>':
                    display += '-';
                    break;
                case '@':
                    display += ' ';
                    break;
                case 'K':
                    display += 'H';
                    break;
                case 'M':
                    display += 'N';
                    break;
                case 'Q':
                    display += 'O';
                    break;
                case 'V':
                    display += 'U';
                    break;
                case 'W':
                    display += 'I';
                    break;
                case 'X':
                    display += 'C';
                    break;
                case 'Z':
                    display += 'C';
                    break;
                case '[':
                    display += 'R';
                    break;
                default:
                    display += b;
                    break;
            }
        }
        return display;
    }

    public String varVal(String command, boolean printDS) {
        k2cmds++;
        setChanged();
        notifyObservers(new Integer((int) (100. * k2cmds / k2cmdsTot)));
        if (stopReading) {
            return "";
        }
        sport.serialWrite(command.getBytes(), 300);
        byte response[] = new byte[16];
        sport.serialRead(response);
        if (printDS) {
            // Get response from rig and decode it.
            sport.serialWrite("DS;".getBytes(), 300);
            response = new byte[16];
            return parseDS(sport.serialRead(response));
        }
        return null;
    }

    public String varVal(String command, boolean printDS, int delay) {
        k2cmds++;
        setChanged();
        notifyObservers(new Integer((int) (100. * k2cmds / k2cmdsTot)));
        if (stopReading) {
            return "";
        }
        sport.serialWrite(command.getBytes(), 300);
        byte response[] = new byte[16];
        sport.serialRead(response);
        try {
            Thread.sleep(delay);
        } catch (InterruptedException e) {
        }
//        String chk = new String(response);
//        if (chk.trim().isEmpty()) {
//            stopReading = true;
//            JOptionPane.showMessageDialog(null,
//                    "No response from K2.",
//                    "K2 communication error",
//                    JOptionPane.ERROR_MESSAGE);
//            return "";
//        }
        if (printDS) {
            // Get response from rig and decode it.
            sport.serialWrite("DS;".getBytes(), 300);
            response = new byte[32];
            return parseDS(sport.serialRead(response));
        }
        return null;
    }

    private void qWriteln(BufferedWriter out, String s) {
        qWrite(out, s + "\n");
    }

    private void qWrite(BufferedWriter out, String s) {
        try {
            out.write(s);
            // Write freq/swr values to file.
        } catch (IOException ex) {
            System.out.println("Couldn't write to "
                    + outputFile.getAbsoluteFile());
        }
    }

    @Override
    public void run() {
        readSettings();
    }
}
